close all; clear all; clc;
format compact

global NMVT NMEDG NMEL MCOORD MVERT MEDGE MADJ
%==========================================================
%     set data of the  problem
%==========================================================

%  definition of the problem data
%  (function handles stored as 'global')
global  ffct ufct 
example_ID = 2
examples_heat (example_ID);

%  definition of macro-grid + bd_part information
[bd_parts, macro_grid] = domain_bfs ();

%  plot the macro grid
fig_nr = figure;
plotgrid (NMEL, NMEDG, NMVT, MCOORD, MVERT, MEDGE, MADJ, fig_nr);

%==========================================================
return
%==========================================================

% ... maximum number of grid levels
%     (level 1 corresponds to the macro-grid)
level = 4
fe_order = 1

switch (fe_order)
  case 1
    %... for Q1-elements
    basis = @basis_q1
    get_dof = @get_dof_Q1
    get_bd_dof = @get_bd_dof_Q1
    nloc = 4;   % nr. of local dof's per cell 
    ngauss = 2 % nr. of Gauss-points per cell (in one direction)
  case 2
    %... for Q2-elements
    basis = @basis_q2
    get_dof = @get_dof_Q2
    get_bd_dof = @get_bd_dof_Q2
    nloc = 9;   % nr. of local dof's per cell 
    ngauss = 3 % nr. of Gauss-points per cell (in one direction)
  otherwise
    error('this  "fe_order" is not implemented');
end; % switch  (fe_order)

% plotting f(x,y)
figure(level+1)
[X,Y] = meshgrid(-2:0.2:4,  -1:0.1:1);
Z = ffct(X,Y);
surf(X,Y,Z)
%==========================================================

%  create additional boundary informations
[NMBDEDG, MBDEDG, MELEDG] = domain_bd_data (macro_grid);

% stiffness matrix, rhs vector
global A  F;

% ...  'mesh' is a cell-array
%       mesh(L) holds grid-data of level L
mesh = cell(level,1);

% ...  cell of finite element solutions on all levels 
sol = cell(level, 1);
dofs = cell(level, 1);

% ... number of Dirichlet-boundary nodes
n_bd_dof = zeros(level,1);
u_20 = zeros(level,1);

n_dof = zeros(level,1);

%==========================================================
%     calculation on  macro-grid (level 1)
%==========================================================
mesh{1} = macro_grid;

[nglob,dof] = get_dof (mesh{1});
n_dof(1) = nglob;

% ... assembling
[A, F] = Galerkin_Laplace_F (ffct, nglob, nloc, basis, dof, ngauss,...
                             mesh{1}.NEL, mesh{1}.KVERT,  mesh{1}.KCOORD);


n_bd_dof(1) = set_bd_data(nglob, dof, get_bd_dof, bd_parts, ...
            NMBDEDG, MBDEDG, MELEDG, ...
	    mesh{1}.KVERT, mesh{1}.KEDGE, mesh{1}.KCOORD, mesh{1}.KADJ);

u = A\F; 

sol{1} = u;

%==========================================================
%     loop over all higher levels
%==========================================================

if level <= 1
  error('no refinement');
end;

for L = 2:level   % L_loop

  % ... refine the previous mesh on level L-1
  mesh{L} = refine ( mesh{L-1} );

  %  hold boundary informations
  [NBDEDG, KBDEDG, KELEDG] = domain_bd_data (mesh{L});

  [nglob,dof] = get_dof (mesh{L});
  n_dof(L) = nglob;
  dofs{L} = dof;

  % ... assembling  
  tic
  [A,F] = Galerkin_Laplace_F (ffct, nglob, nloc, basis, dof, ngauss,...
                              mesh{L}.NEL, mesh{L}.KVERT,  mesh{L}.KCOORD);

  % ... incorporating boundary conditions into A and F 
  n_bd_dof(L) = set_bd_data (nglob, dof, get_bd_dof, bd_parts, ... 
                 NBDEDG, KBDEDG, KELEDG, mesh{L}.KVERT, mesh{L}.KEDGE, ...
		     mesh{L}.KCOORD, mesh{L}.KADJ);

  fprintf(' Level = %2i  t_ass = %10.3e  \n',L,toc)
  u = A\F; 

  sol{L} = u;

  % ... compute value u_h(x) at x=(2,0)
%   ivert = 0;
%   for j  = 1: mesh{L}.NVT
%     if norm([2,0]-mesh{L}.KCOORD(j,:)) < 1e-13
%       ivert = j;
%     end
%   end
%   if ivert == 0
%     error('  ivert == 0  ')
%   end
%   u_20(L) = u(ivert);

%  function errL2 = L2_error (ufct, uvec, mesh, ngauss, nloc, basis, dof)
%errL2(L) =          L2_error (ufct, u, mesh{L}, ngauss, nloc , basis, dof);
errL2(L) =          L2_error (ufct, u, mesh{L}, ngauss, nloc , basis, dof);
   
end;   % L_loop
%==========================================================

%  statistical output for all levels
fprintf('\n-----------------------------------------------------------\n');
fprintf('level    n_dof    n_bd_dof    L2-error   \n');
fprintf('-----------------------------------------------------------\n');

for L = 2:level   % L_loop
  fprintf('%1i      %5i        %5i               %15.6e     \n',...
             L,   n_dof(L), n_bd_dof(L), errL2(L)  );
end;   % L_loop

%==========================================================
%  visualization of the FE function u
%==========================================================

for L = 2:level     %  L_loop

%  get solution on level L
u = sol{L};
dof = dofs{L};

nK = 1;    % number of visualization points on each element

stitle = sprintf('plot of u_h (Q_1 elements), level = %1i, dim(V_h) = %4i',...
                                              L,       n_dof(L) );

switch_shading = 0;  % no shading in visualization
visu = visualize (u, dof, basis, macro_grid, mesh{L}, L, nK, stitle, L, switch_shading);
set(gcf,'Position', [1 1 1072 428]);

colorbar;  % add a bar to figure showing the scales of colors

view([0,90]);
axis equal;

% ... save figure as png-file
fig_file =  sprintf( 'heat_backstep_L_%1i.png',L); % file-name of figure-file
print( '-dpng',  fig_file);

%fig_file =  sprintf( 'heat_backstep_L_%1i.eps',L); % file-name of figure-file
%print( '-depsc',  fig_file);

end   % L_loop

%==========================================================
format 

